﻿using UnityEngine;
using Steamworks;
using HarmonyLib;

public class RadioSync : Mod
{
    static RadioSync instance;
    private static Raft_Network network = ComponentManager<Raft_Network>.Value;
    private static bool radiosOn = false;

    public void Start()
    {
        instance = this;
        Debug.Log("Mod RadioSync has been loaded!");
    }

    public void OnModUnload()
    {
        Debug.Log("Mod RadioSync has been unloaded!");
    }

    public void toggleRadios(int cassetIndex = -1, int timeline = 0)
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        if (cassetIndex > -1)
        {
            setRadiosWithCassette(cassetIndex);
        }
        RadioPlayer[] radios = FindObjectsOfType<RadioPlayer>();
        for (int i = 0; i < radios.Length; ++i)
        {
            Message_RadioPlayer_Toggle message = new Message_RadioPlayer_Toggle(Messages.Radio_Toggle, RAPI.GetLocalPlayer().Network.NetworkIDManager, radios[i].ObjectIndex, timeline);
            if (Raft_Network.IsHost)
            {
                AccessTools.Method("RadioPlayer:ToggleRadio", null, null).Invoke(radios[i], null);
                network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                continue;
            }
            network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
        }
    }

    public void setRadiosWithCassette(int cassetIndex, int timeline = 0)
    {
        if (network == null)
            network = ComponentManager<Raft_Network>.Value;
        RadioPlayer[] radios = FindObjectsOfType<RadioPlayer>();
        for (int i = 0; i < radios.Length; ++i)
        {
            Message_InteractableButtonPress_Integer msg = new Message_InteractableButtonPress_Integer(RAPI.GetLocalPlayer().Network.NetworkIDManager, radios[i].ObjectIndex, 0, RAPI.GetLocalPlayer(), cassetIndex);
            if (Raft_Network.IsHost)
            {
                RGD_RadioPlayer rgd = new RGD_RadioPlayer();
                rgd.cassetteIndex = cassetIndex;
                rgd.timelinePosition = timeline;
                radios[i].RestoreRadio(rgd);
                network.RPC(msg, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                continue;
            }
            network.SendP2P(network.HostID, msg, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
        }
        if (timeline > 0)
        {
            syncRadios(timeline);
        }
    }

    public void syncRadios(int value = 0)
    {
        Debug.Log("Syncing radios to timeline value:" + value);
        RadioPlayer[] radios = FindObjectsOfType<RadioPlayer>(); 
        
        for (int i = 0; i < radios.Length; ++i)
        {
            Message_RadioPlayer_Toggle message = new Message_RadioPlayer_Toggle(Messages.Radio_Toggle, RAPI.GetLocalPlayer().Network.NetworkIDManager, radios[i].ObjectIndex, value);
            if (Raft_Network.IsHost)
            {
                radios[i].GetCassetConnectionIndexFromIndex(radios[i].CassetteIndex).timelinePosition = value;
                radios[i].GetCassetConnectionIndexFromIndex(radios[i].CassetteIndex).SetEmitterToCurrentTimeline();
                radios[i].GetCassetConnectionIndexFromIndex(radios[i].CassetteIndex).SaveTimelinePosition();
                radios[i].Serialize_Save();
                radios[i].ToggleRadio();
                network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                if(!radios[i].isActiveAndEnabled)
                {
                    radios[i].ToggleRadio();
                    network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                }
                continue;
            }
            network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game); 
            if (!radios[i].isActiveAndEnabled)
            {
                radios[i].ToggleRadio();
                network.SendP2P(network.HostID, message, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
            }
        }
    }


    [ConsoleCommand(name: "radioInfo", docs: "reads all cassette indexes")]
    public static void info_gather(string[] args)
    {
        RadioPlayer[] radios = FindObjectsOfType<RadioPlayer>();
        foreach (RadioPlayer radio in radios)
        {
            Debug.Log(radio.ToString() + "\tCassette: " + radio.CassetteIndex);
        }
    }

    //[ConsoleCommand(name: "syncRadios", docs: "Toggle all Radios ON/OFF at once")]
    //public static void syncRadiosCommand(string[] args)
    //{
    //    int value = 0;
    //    if (args.Length > 0)
    //        int.TryParse(args[0], out value);
    //    instance.syncRadios(value);
    //}

    [ConsoleCommand(name: "setCassettes", docs: "Set all Radios with a particular Cassette")]
    public static void setCassettes(string[] args)
    {
        int value = 0;
        if (args.Length > 0)
            int.TryParse(args[0], out value);
        instance.setRadiosWithCassette(value);
    }

    [ConsoleCommand(name: "toggleRadios", docs: "Toggle all Radios ON/OFF at once")]
    public static void toggleRadiosCommand(string[] args)
    {
        int cassette = -1;
        if (args.Length > 0)
            int.TryParse(args[0], out cassette);
        int timeline = 0;
        if (args.Length > 1)
            int.TryParse(args[1], out timeline);
        instance.toggleRadios(cassette, timeline);
    }
}